@@ -21,7 +21,14 @@
 #'                   all of them.
 #' @param ythreshold Indicates the proportion of potential critical y values
 #'                   to display in the output. A value of \code{1} would display
-#'                   all of them.                   
+#'                   all of them.
+#' @param progress If \code{TRUE}, prints an indicator of progress as for loops
+#'                 progress.
+#' @param verbose If \code{FALSE}, suppresses printed output of tables.
+#' @param listout If \code{TRUE}, outputs a list of data frames instead of a
+#'                a single data frame.  This allows a data frame of critical
+#'                values and associated statistics to be extracted, for example
+#'                if one would want to sort by Cramer's V.                             
 #'             
 #' @details  Cate-Nelson analysis divides bivariate data into two groups.
 #'           For data with a positive trend, one group has a
@@ -46,7 +53,9 @@
 #'         the proportion of observations that conform with the model, 
 #'         the number of observations that do not conform to the model,
 #'         the proportion of observations that do not conform to the model,
-#'         and a p-value for the Fisher exact test for the data divided into
+#'         a p-value for the Fisher exact test for the data divided into
+#'         the groups indicated by the model,
+#'         and Cramer's V for the data divided into
 #'         the groups indicated by the model.
 #'         
 #'         Output also includes printed lists of critical values,
@@ -88,7 +97,8 @@
 cateNelson = 
   function(x, y, plotit=TRUE, hollow=TRUE, xlab="X", ylab="Y", 
            trend="positive", clx=1, cly=1, 
-           xthreshold=0.10, ythreshold=0.10)
+           xthreshold=0.10, ythreshold=0.10,
+           progress=TRUE, verbose=TRUE, listout=FALSE)
   {
   
   n = length(x)
@@ -108,7 +118,8 @@ cateNelson =
                            Q.iii=as.integer(rep(0,n)),
                            Q.iv=as.integer(rep(0,n)),
                            Q.model=as.integer(rep(0,n)),
-                           Q.err=as.integer(rep(0,n))
+                           Q.err=as.integer(rep(0,n)),
+                           Cramer.V=as.numeric(rep(0.00, n))
                            )
       dataset = dataset[with(dataset, order(x, y)),]
       
@@ -118,35 +129,43 @@ cateNelson =
       dataset$Critical.x.value[1] = dataset$Critical.x.value[1]/2 
 
   m = n-2
+  if(progress){tick=0}
   for(j in (3:m)){
+    if(progress){tick=tick+1}
+    if(progress){if(tick>(m-3)/100){tick=0;cat(".")}}
       for (i in (1:n)){
          dataset$xgroup[i] = "b"
          if(dataset$x[i] < dataset$Critical.x.value[j])
             dataset$xgroup[i] = as.factor("a")
-         }
+      }
         fit = lm(y ~ xgroup, data=dataset)
         fit1 = anova(fit)
         dataset$Sum.of.squares[j] = (fit1[1,2])
       }
+    if(progress){cat("\n")}
+    
         dataset$Sum.of.squares[1] = min(dataset$Sum.of.squares[3:(n-2)])
         dataset$Sum.of.squares[2] = min(dataset$Sum.of.squares[3:(n-2)])
         dataset$Sum.of.squares[n-1] = min(dataset$Sum.of.squares[3:(n-2)])
         dataset$Sum.of.squares[n] = min(dataset$Sum.of.squares[3:(n-2)])
         max.ss = min(dataset$Sum.of.squares)+
                  (max(dataset$Sum.of.squares)-min(dataset$Sum.of.squares))*
-                 (1-xthreshold)    
-        dataset2 = dataset[dataset$Sum.of.squares >= max.ss,]
-        dataset2 = dataset2[with(dataset2, order(-Sum.of.squares)),]
+                 (1-xthreshold)
+        
+              
+      if(plotit){dataset4 = dataset[2:length(dataset[[1]]),]}
+
+        dataset2 = dataset[with(dataset, order(-Sum.of.squares)),]
         rownames(dataset2) <- 1:nrow(dataset2)
         CLX = dataset2$Critical.x.value[clx]
             for (i in (1:n)){
                dataset$xgroup[i] = if(dataset$x[i] < CLX)  "a" else "b"
                }
   
-  cat("\n") 
+  if(verbose){cat("\n") 
   cat("Critical x that maximize sum of squares:","\n","\n") 
-  print(dataset2[c("Critical.x.value", "Sum.of.squares")])
-   
+  print(dataset2[dataset2$Sum.of.squares >= max.ss,][c("Critical.x.value", "Sum.of.squares")])}
+  
   dataset = dataset[with(dataset, order(y, x)),]
   dataset$Critical.y.value[1] = 0
   for(k in c(2:n)){
@@ -155,7 +174,10 @@ cateNelson =
 
   dataset$Critical.y.value[1] = min(dataset$Critical.y.value[2:n])
 
+  if(progress){tick=0}
   for(j in c(1:n)){
+    if(progress){tick=tick+1}
+    if(progress){if(tick>(m-3)/100){tick=0;cat(".")}}
      for (i in c(1:n)){
        dataset$ygroup[i] = "d"
        if(dataset$y[i] < dataset$Critical.y.value[j])
@@ -171,10 +193,16 @@ cateNelson =
         dataset$q.iv[i] = with(dataset, ifelse
         ((dataset$xgroup[i]=="a" & dataset$ygroup[i]=="c"), 1, 0)) 
        }
-       dataset$Q.i[j] = sum(dataset$q.i)
-       dataset$Q.ii[j] = sum(dataset$q.ii)
+       dataset$Q.i[j]   = sum(dataset$q.i)
+       dataset$Q.ii[j]  = sum(dataset$q.ii)
        dataset$Q.iii[j] = sum(dataset$q.iii)
-       dataset$Q.iv[j] = sum(dataset$q.iv)
+       dataset$Q.iv[j]  = sum(dataset$q.iv)
+       dataset$Cramer.V[j] = as.numeric(cramerV(matrix(c(dataset$Q.i[j],
+                                                         dataset$Q.ii[j],
+                                                         dataset$Q.iv[j],
+                                                         dataset$Q.iii[j]),
+                                                         nrow=2,
+                                                         byrow=TRUE)))
        if (trend=="positive")
        dataset$Q.model[j] = dataset$Q.ii[j] + dataset$Q.iv[j]
        if (trend=="negative")
@@ -183,17 +211,21 @@ cateNelson =
        dataset$Q.err[j] = dataset$Q.i[j] + dataset$Q.iii[j]
        if (trend=="negative")
        dataset$Q.err[j] = dataset$Q.ii[j] + dataset$Q.iv[j] 
-   } 
+     }
+      if(progress){cat("\n")}
      
 min.qerr = min(dataset$Q.err)+
           (max(dataset$Q.err)-min(dataset$Q.err))*(ythreshold)               
-dataset3 = dataset[dataset$Q.err <= min.qerr,]
-dataset3 = dataset3[with(dataset3, order(Q.err)),]
+
+dataset3 = dataset[with(dataset, order(Q.err)),]
+
 rownames(dataset3) <- 1:nrow(dataset3)
-  cat("\n")
+  if(verbose){cat("\n")
   cat("Critical y that minimize errors:","\n","\n")
-  print(dataset3[c("Critical.y.value","Q.i","Q.ii","Q.iii","Q.iv",
-                   "Q.model","Q.err")])
+  print(dataset3[dataset3$Q.err <= min.qerr,][c("Critical.y.value",
+                   "Q.i","Q.ii","Q.iii","Q.iv",
+                   "Q.model","Q.err","Cramer.V")])}
+
 CLY = dataset3$Critical.y.value[cly]
 
  N = 1
@@ -209,26 +241,29 @@ CLY = dataset3$Critical.y.value[cly]
                  p.Model=as.numeric(rep(0.00,N)),
                  Q.Error=as.integer(rep(0.00,N)),
                  p.Error=as.numeric(rep(0.00,N)),
-                 Fisher.p.value=as.numeric(rep(0.00,N))
+                 Fisher.p.value=as.numeric(rep(0.00,N)),
+                 Cramer.V=as.numeric(rep(0.00,N))
                  )
+
   Z$n[1] = n
   Z$CLx[1] = CLX
   Z$SS[1] = dataset2$Sum.of.squares[1]
   Z$CLy[1] = CLY
-  Z$Q.I[1] = dataset3$Q.i[1]
-  Z$Q.II[1] = dataset3$Q.ii[1]
-  Z$Q.III[1] = dataset3$Q.iii[1]
-  Z$Q.IV[1] = dataset3$Q.iv[1]
-  Z$Q.Model[1] = dataset3$Q.model[1]
-  Z$p.Model[1] = dataset3$Q.model[1]/n
-  Z$Q.Error[1] = dataset3$Q.err[1]
-  Z$p.Error[1] = dataset3$Q.err[1]/n
+  Z$Q.I[1] = dataset3$Q.i[cly]
+  Z$Q.II[1] = dataset3$Q.ii[cly]
+  Z$Q.III[1] = dataset3$Q.iii[cly]
+  Z$Q.IV[1] = dataset3$Q.iv[cly]
+  Z$Q.Model[1] = dataset3$Q.model[cly]
+  Z$p.Model[1] = dataset3$Q.model[cly]/n
+  Z$Q.Error[1] = dataset3$Q.err[cly]
+  Z$p.Error[1] = dataset3$Q.err[cly]/n
   
   M=matrix(c(Z$Q.I[1],Z$Q.II[1],Z$Q.IV[1],Z$Q.III[1]),
            nrow=2,
            byrow=TRUE)
            
-  Z$Fisher.p.value = fisher.test(M)$p.value       
+  Z$Fisher.p.value = fisher.test(M)$p.value
+  Z$Cramer.V = dataset3$Cramer.V[cly]
     
     if (plotit)
      {
@@ -255,13 +290,17 @@ CLY = dataset3$Critical.y.value[cly]
     }         
     if (plotit) 
     plot(dataset$x,dataset$y,pch=16,xlab=xlab,ylab=ylab)           
-    if (plotit)         
-    plot(Sum.of.squares~Critical.x.value,data=dataset,
-         pch=16,xlab="Critical-x value",
-         ylab="Sum of squares")
+    if (plotit){
+    plot(Sum.of.squares~Critical.x.value,
+         data=dataset4,
+         xlab="Critical-x value",
+         ylab="Sum of squares")}
     if (plotit) 
     plot(Q.err~Critical.y.value, data=dataset, xlab="Critical-y value",
          ylab="Number of points in error quadrants")
+    if (plotit) 
+    plot(Cramer.V~Critical.y.value, data=dataset, xlab="Critical-y value",
+         ylab="Cramer's V")
     if (plotit)
     {
      plot(dataset$x,dataset$y,pch=pchi,
@@ -283,24 +322,37 @@ CLY = dataset3$Critical.y.value[cly]
            labels="IV")
      }
  
-cat("\n")    
-cat("n       = Number of observations","\n")
-cat("CLx     = Critical value of x","\n")
-cat("SS      = Sum of squares for that critical value of x","\n")
-cat("CLy     = Critical value of y","\n")
-cat("Q       = Number of observations which fall into quadrants I, II, III, IV",
+if(verbose){cat("\n")    
+cat("n         = Number of observations","\n")
+cat("CLx       = Critical value of x","\n")
+cat("SS        = Sum of squares for that critical value of x","\n")
+cat("CLy       = Critical value of y","\n")
+cat("Q         = Number of observations which fall into quadrants I, II, III, IV",
           "\n")
-cat("Q.Model = Total observations which fall into the quadrants predicted by the model",
+cat("Q.Model   = Total observations which fall into the quadrants predicted by the model",
           "\n")
-cat("p.Model = Percent observations which fall into the quadrants predicted by the model",
+cat("p.Model   = Percent observations which fall into the quadrants predicted by the model",
           "\n")
-cat("Q.Error = Observations which do not fall into the quadrants predicted by the model",
+cat("Q.Error   = Observations which do not fall into the quadrants predicted by the model",
           "\n")
-cat("p.Error = Percent observations which do not fall into the quadrants predicted by the model",
+cat("p.Error   = Percent observations which do not fall into the quadrants predicted by the model",
           "\n")
-cat("Fisher  = p-value from Fisher exact test dividing data into these quadrants",
+cat("Fisher.p  = p-value from Fisher exact test dividing data into these quadrants",
+          "\n")
+cat("Cramer.V  = Cramer's V statistic from dividing data into these quadrants",
           "\n")
 cat("\n") 
-cat("Final model:","\n","\n") 
-return(Z)
+cat("Final model:","\n","\n")}
+if(!listout){
+  return(Z)}
+
+  if(listout){
+  print(Z)
+  cat("\n") 
+  Zed = list(Critical.x = dataset2[c("Critical.x.value", "Sum.of.squares")],
+             Critical.y = dataset3[c("Critical.y.value",
+                                     "Q.i","Q.ii","Q.iii","Q.iv",
+                                     "Q.model","Q.err","Cramer.V")],
+             Final.model = Z)
+  return(Zed)}
 }
\ No newline at end of file
